// Fill out your copyright notice in the Description page of Project Settings.

#include "TheracAdapterComponent.h"
#include "HAL/PlatformProcess.h"
#include "Interfaces/IPluginManager.h"
#include "Misc/Paths.h"
#include <Internationalization/Internationalization.h>
#include <Internationalization/Text.h>
#include <Misc/MessageDialog.h>
#include <map>
#include <Logging/StructuredLog.h>

// Sets default values for this component's properties
UTheracAdapterComponent::UTheracAdapterComponent() {
  // Set this component to be initialized when the game starts, and to be ticked
  // every frame.  You can turn these features off to improve performance if you
  // don't need them.
  PrimaryComponentTick.bCanEverTick = true;

  // ...
}

void UTheracAdapterComponent::UpdateSimulator(
    ExtCallType ect,
    HsStablePtr wrapped_comms,
    BeamType beam_type,
    CollimatorPosition collimator_position,
    HsInt beam_energy
) {
  if (wrapped_comms == nullptr)
    wrapped_comms = wrappedComms;
  wrap_external_call(
      wrapped_comms,
      ect,
      beam_type,
      collimator_position,
      beam_energy
  );
}

void UTheracAdapterComponent::WrapSimulatorCall(
    EExtCallType ext_call_type,
    EBeamType beam_type,
    ECollimatorPosition collimator_position,
    int32 beam_energy
) {}

// Called when the game starts
void UTheracAdapterComponent::BeginPlay() {

  FString BaseDir =
      IPluginManager::Get().FindPlugin("hstherac25")->GetBaseDir();
  FString LibraryPath =
      FPaths::Combine(*BaseDir, TEXT("Source/hstherac-hs/hstherac25.dll"));

  simulatorLibraryHandle = !LibraryPath.IsEmpty()
                               ? FPlatformProcess::GetDllHandle(*LibraryPath)
                               : nullptr;

  Super::BeginPlay();
  wrappedComms                            = start_machine();
  std::map<FText *, StateInfoRequest> hng = {
      {&TreatmentOutcome, RequestTreatmentOutcome},
      {&ActiveSubsystem, RequestActiveSubsystem},
      {&TreatmentState, RequestTreatmentState},
      {&Reason, RequestReason},
      {&BeamMode, RequestBeamMode},
      {&BeamEnergy, RequestBeamEnergy}
  };
  for (auto & [hnng, hnnng] : hng) {
    compMap.Add(hnng, hnnng);
  }
  ExtCallTypeMap.Add("R", EExtCallType::ExtCallReset);
  ExtCallTypeMap.Add("P", EExtCallType::ExtCallProceed);

  BeamTypeMap.Add("X", EBeamType::BeamTypeXRay);
  BeamTypeMap.Add("E", EBeamType::BeamTypeElectron);

  CollimatorPositionMap.Add("X", ECollimatorPosition::CollimatorPositionXRay);
  CollimatorPositionMap.Add(
      "E",
      ECollimatorPosition::CollimatorPositionElectronBeam
  );

  // ...
}

void UTheracAdapterComponent::shutdownSimulator() {
  kill_machine();
  FPlatformProcess::FreeDllHandle(simulatorLibraryHandle);
  simulatorLibraryHandle = nullptr;
}

// Called every frame
void UTheracAdapterComponent::TickComponent(
    float DeltaTime,
    ELevelTick TickType,
    FActorComponentTickFunction * ThisTickFunction
) {
  Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

  auto rsi = [&](FText * f) {
    return FText::FromString(
        StaticCast<char *>(request_state_info(wrappedComms, *compMap.Find(f)))
    );
  };

  for (auto & [f, _] : compMap) {
    //    UE_LOGFMT(LogTemp, Warning, "what {dildo}", rsi(f).ToString());
    *f = rsi(f);
  }
}

